from typing import Dict, List

from s3transfer.tasks import CompleteMultipartUploadTask as CompleteMultipartUploadTask
from s3transfer.tasks import CreateMultipartUploadTask as CreateMultipartUploadTask
from s3transfer.tasks import SubmissionTask as SubmissionTask
from s3transfer.tasks import Task as Task
from s3transfer.utils import ChunksizeAdjuster as ChunksizeAdjuster
from s3transfer.utils import calculate_range_parameter as calculate_range_parameter
from s3transfer.utils import get_callbacks as get_callbacks
from s3transfer.utils import get_filtered_dict as get_filtered_dict

class CopySubmissionTask(SubmissionTask):
    EXTRA_ARGS_TO_HEAD_ARGS_MAPPING: Dict[str, str]
    UPLOAD_PART_COPY_ARGS: List[str]
    CREATE_MULTIPART_ARGS_BLACKLIST: List[str]
    COMPLETE_MULTIPART_ARGS: List[str]

class CopyObjectTask(Task): ...
class CopyPartTask(Task): ...



import logging
from typing import (
    IO,
    Any,
    Callable,
    Dict,
    Iterable,
    Iterator,
    List,
    Optional,
    Sequence,
    Tuple,
    Type,
    TypeVar,
    Union,
)

from botocore.awsrequest import AWSRequest
from s3transfer.compat import SOCKET_ERROR as SOCKET_ERROR
from s3transfer.compat import fallocate as fallocate
from s3transfer.compat import rename_file as rename_file
from s3transfer.futures import TaskTag, TransferFuture

_R = TypeVar("_R")

MAX_PARTS: int
MAX_SINGLE_UPLOAD_SIZE: int
MIN_UPLOAD_CHUNKSIZE: int
logger: logging.Logger
S3_RETRYABLE_DOWNLOAD_ERRORS: Tuple[Type[BaseException], ...]

def random_file_extension(num_digits: int = ...) -> str: ...
def signal_not_transferring(request: AWSRequest, operation_name: str, **kwargs: Any) -> None: ...
def signal_transferring(request: AWSRequest, operation_name: str, **kwargs: Any) -> None: ...
def calculate_num_parts(size: int, part_size: int) -> int: ...
def calculate_range_parameter(
    part_size: int, part_index: int, num_parts: int, total_size: Optional[int] = ...
) -> str: ...
def get_callbacks(
    transfer_future: TransferFuture, callback_type: str
) -> List[Callable[..., Any]]: ...
def invoke_progress_callbacks(
    callbacks: Iterable[Callable[..., Any]], bytes_transferred: int
) -> None: ...
def get_filtered_dict(
    original_dict: Dict[str, Any], whitelisted_keys: Sequence[str]
) -> Dict[str, Any]: ...

class CallArgs:
    def __init__(self, **kwargs: Any) -> None: ...

class FunctionContainer:
    def __init__(self, func: Callable[..., Any], *args: Any, **kwargs: Any) -> None: ...
    def __call__(self) -> Any: ...

class CountCallbackInvoker:
    def __init__(self, callback: Callable[..., Any]) -> None: ...
    @property
    def current_count(self) -> int: ...
    def increment(self) -> None: ...
    def decrement(self) -> None: ...
    def finalize(self) -> None: ...

class OSUtils:
    def get_file_size(self, filename: str) -> int: ...
    def open_file_chunk_reader(
        self, filename: str, start_byte: int, size: int, callbacks: Iterable[Callable[..., Any]]
    ) -> ReadFileChunk: ...
    def open_file_chunk_reader_from_fileobj(
        self,
        fileobj: Union[IO[Any], str, bytes],
        chunk_size: int,
        full_file_size: int,
        callbacks: Iterable[Callable[..., Any]],
        close_callbacks: Optional[Iterable[Callable[..., Any]]] = ...,
    ) -> ReadFileChunk: ...
    def open(self, filename: str, mode: str) -> IO[Any]: ...
    def remove_file(self, filename: str) -> None: ...
    def rename_file(self, current_filename: str, new_filename: str) -> None: ...
    # FIXME: Instance methods should take a "self" parameter
    def is_special_file(cls, filename: str) -> bool: ...  # type: ignore
    def get_temp_filename(self, filename: str) -> str: ...
    def allocate(self, filename: str, size: int) -> None: ...

class DeferredOpenFile:
    def __init__(
        self,
        filename: str,
        start_byte: int = ...,
        mode: str = ...,
        open_function: Callable[..., Any] = ...,
    ) -> None: ...
    @property
    def name(self) -> str: ...
    def read(self, amount: Optional[int] = ...) -> bytes: ...
    def write(self, data: str) -> None: ...
    def seek(self, where: int, whence: int = ...) -> None: ...
    def tell(self) -> int: ...
    def close(self) -> None: ...
    def __enter__(self: _R) -> _R: ...
    def __exit__(self, *args: Any, **kwargs: Any) -> None: ...

class ReadFileChunk:
    def __init__(
        self,
        fileobj: Union[IO[Any], str, bytes],
        chunk_size: int,
        full_file_size: int,
        callbacks: Optional[Iterable[Callable[..., Any]]] = ...,
        enable_callbacks: bool = ...,
        close_callbacks: Optional[Iterable[Callable[..., Any]]] = ...,
    ) -> None: ...
    @classmethod
    def from_filename(
        cls: Type[_R],
        filename: str,
        start_byte: int,
        chunk_size: int,
        callbacks: Optional[Iterable[Callable[..., Any]]] = ...,
        enable_callbacks: bool = ...,
    ) -> _R: ...
    def read(self, amount: Optional[int] = ...) -> bytes: ...
    def signal_transferring(self) -> None: ...
    def signal_not_transferring(self) -> None: ...
    def enable_callback(self) -> None: ...
    def disable_callback(self) -> None: ...
    def seek(self, where: int, whence: int = ...) -> None: ...
    def close(self) -> None: ...
    def tell(self) -> int: ...
    def __len__(self) -> int: ...
    def __enter__(self: _R) -> _R: ...
    def __exit__(self, *args: Any, **kwargs: Any) -> None: ...
    def __iter__(self) -> Iterator[bytes]: ...

class StreamReaderProgress:
    def __init__(
        self, stream: IO[Any], callbacks: Optional[Iterable[Callable[..., Any]]] = ...
    ) -> None: ...
    def read(self, *args: Any, **kwargs: Any) -> bytes: ...

class NoResourcesAvailable(Exception): ...

class TaskSemaphore:
    def __init__(self, count: int) -> None: ...
    def acquire(self, tag: TaskTag, blocking: bool = ...) -> int: ...
    def release(self, tag: TaskTag, acquire_token: int) -> None: ...

class SlidingWindowSemaphore(TaskSemaphore):
    def __init__(self, count: int) -> None: ...
    def current_count(self) -> int: ...
    def acquire(self, tag: TaskTag, blocking: bool = ...) -> int: ...
    def release(self, tag: TaskTag, acquire_token: int) -> None: ...

class ChunksizeAdjuster:
    max_size: int
    min_size: int
    max_parts: int
    def __init__(self, max_size: int = ..., min_size: int = ..., max_parts: int = ...) -> None: ...
    def adjust_chunksize(self, current_chunksize: int, file_size: Optional[int] = ...) -> int: ...



from concurrent.futures import CancelledError as CancelledError

class RetriesExceededError(Exception):
    last_exception: BaseException
    def __init__(self, last_exception: BaseException, msg: str = ...) -> None: ...

class S3UploadFailedError(Exception): ...
class InvalidSubscriberMethodError(Exception): ...
class TransferNotDoneError(Exception): ...
class FatalError(CancelledError): ...



from typing import IO, Any, Callable, Iterator, List, Optional, Sequence, Tuple, TypeVar, Union

from s3transfer.bandwidth import BandwidthLimiter
from s3transfer.compat import readable as readable
from s3transfer.compat import seekable as seekable
from s3transfer.futures import IN_MEMORY_UPLOAD_TAG as IN_MEMORY_UPLOAD_TAG
from s3transfer.futures import TransferCoordinator, TransferFuture
from s3transfer.manager import TransferConfig
from s3transfer.tasks import CompleteMultipartUploadTask as CompleteMultipartUploadTask
from s3transfer.tasks import CreateMultipartUploadTask as CreateMultipartUploadTask
from s3transfer.tasks import SubmissionTask as SubmissionTask
from s3transfer.tasks import Task as Task
from s3transfer.utils import ChunksizeAdjuster as ChunksizeAdjuster
from s3transfer.utils import DeferredOpenFile as DeferredOpenFile
from s3transfer.utils import OSUtils, ReadFileChunk
from s3transfer.utils import get_callbacks as get_callbacks
from s3transfer.utils import get_filtered_dict as get_filtered_dict

_R = TypeVar("_R")

class AggregatedProgressCallback:
    def __init__(self, callbacks: Sequence[Callable[..., Any]], threshold: int = ...) -> None: ...
    def __call__(self, bytes_transferred: int) -> None: ...
    def flush(self) -> None: ...

class InterruptReader:
    def __init__(
        self, fileobj: Union[IO[Any], str, bytes], transfer_coordinator: TransferCoordinator
    ) -> None: ...
    def read(self, amount: Optional[int] = ...) -> str: ...
    def seek(self, where: int, whence: int = ...) -> None: ...
    def tell(self) -> int: ...
    def close(self) -> None: ...
    def __enter__(self: _R) -> _R: ...
    def __exit__(self, *args: Any, **kwargs: Any) -> None: ...

class UploadInputManager:
    def __init__(
        self,
        osutil: OSUtils,
        transfer_coordinator: TransferCoordinator,
        bandwidth_limiter: Optional[BandwidthLimiter] = ...,
    ) -> None: ...
    @classmethod
    def is_compatible(cls, upload_source: Any) -> bool: ...
    def stores_body_in_memory(self, operation_name: str) -> bool: ...
    def provide_transfer_size(self, transfer_future: TransferFuture) -> None: ...
    def requires_multipart_upload(
        self, transfer_future: TransferFuture, config: TransferConfig
    ) -> bool: ...
    def get_put_object_body(self, transfer_future: TransferFuture) -> bytes: ...
    def yield_upload_part_bodies(
        self, transfer_future: TransferFuture, chunksize: int
    ) -> Iterator[Tuple[int, ReadFileChunk]]: ...

class UploadFilenameInputManager(UploadInputManager):
    @classmethod
    def is_compatible(cls, upload_source: Any) -> bool: ...
    def stores_body_in_memory(self, operation_name: str) -> bool: ...
    def provide_transfer_size(self, transfer_future: TransferFuture) -> None: ...
    def requires_multipart_upload(
        self, transfer_future: TransferFuture, config: TransferConfig
    ) -> bool: ...
    def get_put_object_body(self, transfer_future: TransferFuture) -> bytes: ...
    def yield_upload_part_bodies(
        self, transfer_future: TransferFuture, chunksize: int
    ) -> Iterator[Tuple[int, ReadFileChunk]]: ...

class UploadSeekableInputManager(UploadFilenameInputManager):
    @classmethod
    def is_compatible(cls, upload_source: Any) -> bool: ...
    def stores_body_in_memory(self, operation_name: str) -> bool: ...
    def provide_transfer_size(self, transfer_future: TransferFuture) -> None: ...

class UploadNonSeekableInputManager(UploadInputManager):
    def __init__(
        self,
        osutil: OSUtils,
        transfer_coordinator: TransferCoordinator,
        bandwidth_limiter: Optional[BandwidthLimiter] = ...,
    ) -> None: ...
    @classmethod
    def is_compatible(cls, upload_source: Any) -> bool: ...
    def stores_body_in_memory(self, operation_name: str) -> bool: ...
    def provide_transfer_size(self, transfer_future: TransferFuture) -> None: ...
    def requires_multipart_upload(
        self, transfer_future: TransferFuture, config: TransferConfig
    ) -> bool: ...
    def get_put_object_body(self, transfer_future: TransferFuture) -> bytes: ...
    def yield_upload_part_bodies(
        self, transfer_future: TransferFuture, chunksize: int
    ) -> Iterator[Tuple[int, ReadFileChunk]]: ...

class UploadSubmissionTask(SubmissionTask):
    UPLOAD_PART_ARGS: List[str]
    COMPLETE_MULTIPART_ARGS: List[str]

class PutObjectTask(Task): ...
class UploadPartTask(Task): ...



import logging
from io import BytesIO
from typing import (
    IO,
    Any,
    Callable,
    Dict,
    Iterator,
    Mapping,
    Optional,
    Sequence,
    Type,
    TypeVar,
    Union,
)

# FIXME: awscrt is untyped
from awscrt.auth import AwsCredentials  # type: ignore
from awscrt.http import HttpRequest  # type: ignore
from awscrt.s3 import S3Client, S3Request  # type: ignore
from botocore.credentials import CredentialProvider
from botocore.session import Session
from s3transfer.constants import GB as GB
from s3transfer.constants import MB as MB
from s3transfer.exceptions import TransferNotDoneError as TransferNotDoneError
from s3transfer.futures import BaseTransferFuture as BaseTransferFuture
from s3transfer.futures import BaseTransferMeta as BaseTransferMeta
from s3transfer.subscribers import BaseSubscriber
from s3transfer.utils import CallArgs as CallArgs
from s3transfer.utils import OSUtils as OSUtils
from s3transfer.utils import get_callbacks as get_callbacks

_R = TypeVar("_R")

logger: logging.Logger

class CRTCredentialProviderAdapter:
    def __init__(self, botocore_credential_provider: CredentialProvider) -> None: ...
    def __call__(self) -> AwsCredentials: ...

def create_s3_crt_client(
    region: str,
    botocore_credential_provider: Optional[CredentialProvider] = ...,
    num_threads: Optional[int] = ...,
    target_throughput: float = ...,
    part_size: int = ...,
    use_ssl: bool = ...,
    verify: Optional[Union[bool, str]] = ...,
) -> S3Client: ...

class CRTTransferManager:
    def __init__(
        self,
        crt_s3_client: S3Client,  # type: ignore
        crt_request_serializer: BaseCRTRequestSerializer,
        osutil: Optional[OSUtils] = ...,
    ) -> None: ...
    def __enter__(self: _R) -> _R: ...
    def __exit__(
        self, exc_type: Type[BaseException], exc_value: BaseException, *args: Any
    ) -> None: ...
    def download(
        self,
        bucket: str,
        key: str,
        fileobj: Union[IO[Any], str, bytes],
        extra_args: Optional[Mapping[str, Any]] = ...,
        subscribers: Optional[Sequence[BaseSubscriber]] = ...,
    ) -> CRTTransferFuture: ...
    def upload(
        self,
        fileobj: Union[IO[Any], str, bytes],
        bucket: str,
        key: str,
        extra_args: Optional[Mapping[str, Any]] = ...,
        subscribers: Optional[Sequence[BaseSubscriber]] = ...,
    ) -> CRTTransferFuture: ...
    def delete(
        self,
        bucket: str,
        key: str,
        extra_args: Optional[Mapping[str, Any]] = ...,
        subscribers: Optional[Sequence[BaseSubscriber]] = ...,
    ) -> CRTTransferFuture: ...
    def shutdown(self, cancel: bool = ...) -> None: ...

class CRTTransferMeta(BaseTransferMeta):
    def __init__(
        self, transfer_id: Optional[str] = ..., call_args: Optional[Mapping[str, Any]] = ...
    ) -> None: ...
    @property
    def call_args(self) -> Dict[str, Any]: ...
    @property
    def transfer_id(self) -> str: ...
    @property
    def user_context(self) -> Dict[str, Any]: ...

class CRTTransferFuture(BaseTransferFuture):
    def __init__(
        self,
        meta: Optional[CRTTransferMeta] = ...,
        coordinator: Optional[CRTTransferCoordinator] = ...,
    ) -> None: ...
    @property
    def meta(self) -> CRTTransferMeta: ...
    def done(self) -> bool: ...
    # FIXME: Signature of "result" incompatible with supertype "BaseTransferFuture"
    def result(self, timeout: Optional[float] = ...) -> None: ...  # type: ignore
    def cancel(self) -> None: ...
    def set_exception(self, exception: BaseException) -> None: ...

class BaseCRTRequestSerializer:
    def serialize_http_request(self, transfer_type: str, future: CRTTransferFuture) -> None: ...

class BotocoreCRTRequestSerializer(BaseCRTRequestSerializer):
    def __init__(
        self, session: Session, client_kwargs: Optional[Mapping[str, Any]] = ...
    ) -> None: ...
    def serialize_http_request(
        self, transfer_type: str, future: CRTTransferFuture
    ) -> HttpRequest: ...

class FakeRawResponse(BytesIO):
    def stream(self, amt: int = ..., decode_content: Optional[bool] = ...) -> Iterator[bytes]: ...

class CRTTransferCoordinator:
    def __init__(
        self,
        transfer_id: Optional[str] = ...,
        s3_request: Optional[S3Request] = ...,  # type: ignore
    ) -> None:
        self.transfer_id: str
    @property
    def s3_request(self) -> S3Request: ...
    def set_done_callbacks_complete(self) -> None: ...
    def wait_until_on_done_callbacks_complete(self, timeout: Optional[float] = ...) -> None: ...
    def set_exception(self, exception: BaseException, override: bool = ...) -> None: ...
    def cancel(self) -> None: ...
    def result(self, timeout: Optional[float] = ...) -> None: ...
    def done(self) -> bool: ...
    def set_s3_request(
        self,
        s3_request: S3Request,  # type: ignore
    ) -> None: ...

class S3ClientArgsCreator:
    def __init__(
        self, crt_request_serializer: BaseCRTRequestSerializer, os_utils: OSUtils
    ) -> None: ...
    def get_make_request_args(
        self,
        request_type: str,
        call_args: Mapping[str, Any],
        coordinator: CRTTransferCoordinator,
        future: CRTTransferFuture,
        on_done_after_calls: Sequence[Callable[..., Any]],
    ) -> Dict[str, Any]: ...
    def get_crt_callback(
        self,
        future: CRTTransferFuture,
        callback_type: str,
        before_subscribers: Optional[Sequence[BaseSubscriber]] = ...,
        after_subscribers: Optional[Sequence[BaseSubscriber]] = ...,
    ) -> Callable[..., Any]: ...

class RenameTempFileHandler:
    def __init__(
        self,
        coordinator: CRTTransferCoordinator,
        final_filename: str,
        temp_filename: str,
        osutil: OSUtils,
    ) -> None: ...
    def __call__(self, **kwargs: Any) -> None: ...

class AfterDoneHandler:
    def __init__(self, coordinator: CRTTransferCoordinator) -> None: ...
    def __call__(self, **kwargs: Any) -> None: ...



import logging
from typing import Any, Callable, Dict, List, Mapping, NamedTuple, Optional, Type

from s3transfer.compat import MAXINT as MAXINT
from s3transfer.exceptions import TransferNotDoneError as TransferNotDoneError
from s3transfer.tasks import Task
from s3transfer.utils import FunctionContainer as FunctionContainer
from s3transfer.utils import TaskSemaphore as TaskSemaphore

logger: logging.Logger

class BaseTransferFuture:
    @property
    def meta(self) -> Any: ...
    def done(self) -> bool: ...
    def result(self) -> str: ...
    def cancel(self) -> None: ...

class BaseTransferMeta:
    @property
    def call_args(self) -> Dict[str, Any]: ...
    @property
    def transfer_id(self) -> str: ...
    @property
    def user_context(self) -> Dict[str, Any]: ...

class TransferFuture(BaseTransferFuture):
    def __init__(
        self, meta: Optional[TransferMeta] = ..., coordinator: Optional[TransferCoordinator] = ...
    ) -> None: ...
    @property
    def meta(self) -> TransferMeta: ...
    def done(self) -> bool: ...
    def result(self) -> str: ...
    def cancel(self) -> None: ...
    def set_exception(self, exception: BaseException) -> None: ...

class TransferMeta(BaseTransferMeta):
    def __init__(
        self, call_args: Optional[Mapping[str, Any]] = ..., transfer_id: Optional[str] = ...
    ) -> None: ...
    @property
    def call_args(self) -> Dict[str, Any]: ...
    @property
    def transfer_id(self) -> str: ...
    @property
    def size(self) -> int: ...
    @property
    def user_context(self) -> Dict[str, Any]: ...
    def provide_transfer_size(self, size: int) -> None: ...

class TransferCoordinator:
    transfer_id: str
    def __init__(self, transfer_id: Optional[str] = ...) -> None: ...
    @property
    def exception(self) -> BaseException: ...
    @property
    def associated_futures(self) -> List[TransferFuture]: ...
    @property
    def failure_cleanups(self) -> List[TransferFuture]: ...
    @property
    def status(self) -> str: ...
    def set_result(self, result: str) -> None: ...
    def set_exception(self, exception: BaseException, override: bool = ...) -> None: ...
    def result(self) -> str: ...
    def cancel(self, msg: str = ..., exc_type: Type[BaseException] = ...) -> None: ...
    def set_status_to_queued(self) -> None: ...
    def set_status_to_running(self) -> None: ...
    def submit(
        self, executor: BoundedExecutor, task: Task, tag: Optional[TaskTag] = ...
    ) -> TransferFuture: ...
    def done(self) -> bool: ...
    def add_associated_future(self, future: TransferFuture) -> None: ...
    def remove_associated_future(self, future: TransferFuture) -> None: ...
    def add_done_callback(
        self, function: Callable[..., Any], *args: Any, **kwargs: Any
    ) -> None: ...
    def add_failure_cleanup(
        self, function: Callable[..., Any], *args: Any, **kwargs: Any
    ) -> None: ...
    def announce_done(self) -> None: ...

class BoundedExecutor:
    EXECUTOR_CLS: Type[BaseExecutor]
    def __init__(
        self,
        max_size: int,
        max_num_threads: int,
        tag_semaphores: Optional[Dict[str, Any]] = ...,
        executor_cls: Optional[Type[BaseExecutor]] = ...,
    ) -> None: ...
    def submit(
        self, task: Task, tag: Optional[TaskTag] = ..., block: bool = ...
    ) -> TransferFuture: ...
    def shutdown(self, wait: bool = ...) -> None: ...

class ExecutorFuture:
    def __init__(self, future: TransferFuture) -> None: ...
    def result(self) -> str: ...
    def add_done_callback(self, fn: Callable[[], Any]) -> None: ...
    def done(self) -> bool: ...

class BaseExecutor:
    def __init__(self, max_workers: Optional[int] = ...) -> None: ...
    def submit(self, fn: Callable[..., Any], *args: Any, **kwargs: Any) -> Any: ...
    def shutdown(self, wait: bool = ...) -> None: ...

class NonThreadedExecutor(BaseExecutor):
    def submit(
        self, fn: Callable[..., Any], *args: Any, **kwargs: Any
    ) -> NonThreadedExecutorFuture: ...
    def shutdown(self, wait: bool = ...) -> None: ...

class NonThreadedExecutorFuture:
    def __init__(self) -> None: ...
    def set_result(self, result: str) -> None: ...
    def set_exception_info(self, exception: BaseException, traceback: Any) -> None: ...
    def result(self, timeout: Optional[float] = ...) -> str: ...
    def done(self) -> bool: ...
    def add_done_callback(self, fn: Callable[..., Any]) -> None: ...

class TaskTag(NamedTuple):
    name: str

IN_MEMORY_UPLOAD_TAG: TaskTag
IN_MEMORY_DOWNLOAD_TAG: TaskTag



import os
from multiprocessing.managers import BaseManager as BaseManager
from typing import IO, Any, Callable, Type, Union

rename_file = os.rename

def accepts_kwargs(func: Callable[..., Any]) -> bool: ...

SOCKET_ERROR: Type[ConnectionError]
MAXINT: None

def seekable(fileobj: Union[IO[Any], str, bytes]) -> bool: ...
def readable(fileobj: Union[IO[Any], str, bytes]) -> bool: ...
def fallocate(fileobj: Union[IO[Any], str, bytes], size: int) -> None: ...



from typing import Any, List, Type, TypeVar

from s3transfer.compat import accepts_kwargs as accepts_kwargs
from s3transfer.exceptions import InvalidSubscriberMethodError as InvalidSubscriberMethodError
from s3transfer.futures import TransferFuture

_R = TypeVar("_R", bound="BaseSubscriber")

class BaseSubscriber:
    VALID_SUBSCRIBER_TYPES: List[str]
    def __new__(cls: Type[_R], *args: Any, **kwargs: Any) -> _R: ...
    def on_queued(self, future: TransferFuture, **kwargs: Any) -> None: ...
    def on_progress(
        self, future: TransferFuture, bytes_transferred: int, **kwargs: Any
    ) -> None: ...
    def on_done(self, future: TransferFuture, **kwargs: Any) -> None: ...



import logging
from queue import Queue
from typing import IO, Any, Callable, Dict, Iterator, List, Mapping, Optional, Type, TypeVar, Union

from botocore.awsrequest import AWSRequest
from botocore.client import BaseClient
from s3transfer.exceptions import RetriesExceededError as RetriesExceededError
from s3transfer.exceptions import S3UploadFailedError as S3UploadFailedError
from s3transfer.futures import BaseExecutor

_R = TypeVar("_R")

class NullHandler(logging.Handler):
    def emit(self, record: Any) -> None: ...

logger: logging.Logger = ...
MB: int = ...
SHUTDOWN_SENTINEL: object = ...

def random_file_extension(num_digits: int = ...) -> str: ...
def disable_upload_callbacks(request: AWSRequest, operation_name: str, **kwargs: Any) -> None: ...
def enable_upload_callbacks(request: AWSRequest, operation_name: str, **kwargs: Any) -> None: ...

class QueueShutdownError(Exception): ...

class ReadFileChunk:
    def __init__(
        self,
        fileobj: Union[IO[Any], str, bytes],
        start_byte: int,
        chunk_size: int,
        full_file_size: int,
        callback: Optional[Callable[..., Any]] = ...,
        enable_callback: bool = ...,
    ) -> None: ...
    @classmethod
    def from_filename(
        cls: Type[_R],
        filename: str,
        start_byte: int,
        chunk_size: int,
        callback: Optional[Callable[..., Any]] = ...,
        enable_callback: bool = ...,
    ) -> _R: ...
    def read(self, amount: Optional[int] = ...) -> str: ...
    def enable_callback(self) -> None: ...
    def disable_callback(self) -> None: ...
    def seek(self, where: int) -> None: ...
    def close(self) -> None: ...
    def tell(self) -> int: ...
    def __len__(self) -> int: ...
    def __enter__(self: _R) -> _R: ...
    def __exit__(self, *args: Any, **kwargs: Any) -> None: ...
    def __iter__(self) -> Iterator[str]: ...

class StreamReaderProgress:
    def __init__(self, stream: Any, callback: Optional[Callable[..., Any]] = ...) -> None: ...
    def read(self, *args: Any, **kwargs: Any) -> str: ...

class OSUtils:
    def get_file_size(self, filename: str) -> int: ...
    def open_file_chunk_reader(
        self, filename: str, start_byte: int, size: int, callback: Callable[..., Any]
    ) -> IO[Any]: ...
    def open(self, filename: str, mode: str) -> IO[Any]: ...
    def remove_file(self, filename: str) -> None: ...
    def rename_file(self, current_filename: str, new_filename: str) -> None: ...

class MultipartUploader:
    UPLOAD_PART_ARGS: List[str]
    def __init__(
        self,
        client: BaseClient,
        config: TransferConfig,
        osutil: OSUtils,
        executor_cls: Type[BaseExecutor] = ...,
    ) -> None: ...
    def upload_file(
        self,
        filename: str,
        bucket: str,
        key: str,
        callback: Callable[..., Any],
        extra_args: Mapping[str, Any],
    ) -> None: ...

class ShutdownQueue(Queue[Any]):
    def trigger_shutdown(self) -> None: ...
    # FIXME: Signature of "put" incompatible with supertype "Queue
    def put(self, item: Any) -> None: ...  # type: ignore

class MultipartDownloader:
    def __init__(
        self,
        client: BaseClient,
        config: TransferConfig,
        osutil: OSUtils,
        executor_cls: Type[BaseExecutor] = ...,
    ) -> None: ...
    def download_file(
        self,
        bucket: str,
        key: str,
        filename: str,
        object_size: int,
        extra_args: Dict[str, Any],
        callback: Optional[Callable[..., Any]] = ...,
    ) -> None: ...

class TransferConfig:
    multipart_threshold: int
    max_concurrency: int
    multipart_chunksize: int
    num_download_attempts: int
    max_io_queue: int
    def __init__(
        self,
        multipart_threshold: int = ...,
        max_concurrency: int = ...,
        multipart_chunksize: int = ...,
        num_download_attempts: int = ...,
        max_io_queue: int = ...,
    ) -> None: ...

class S3Transfer:
    ALLOWED_DOWNLOAD_ARGS: List[str]
    ALLOWED_UPLOAD_ARGS: List[str]
    def __init__(
        self,
        client: BaseClient,
        config: Optional[TransferConfig] = ...,
        osutil: Optional[OSUtils] = ...,
    ) -> None: ...
    def upload_file(
        self,
        filename: str,
        bucket: str,
        key: str,
        callback: Optional[Callable[..., Any]] = ...,
        extra_args: Optional[Dict[str, Any]] = ...,
    ) -> None: ...
    def download_file(
        self,
        bucket: str,
        key: str,
        filename: str,
        extra_args: Optional[Dict[str, Any]] = ...,
        callback: Optional[Callable[..., Any]] = ...,
    ) -> None: ...



import logging
from typing import IO, Any, Callable, Dict, List, Optional, TypeVar, Union

from s3transfer.compat import seekable as seekable
from s3transfer.exceptions import RetriesExceededError as RetriesExceededError
from s3transfer.futures import IN_MEMORY_DOWNLOAD_TAG as IN_MEMORY_DOWNLOAD_TAG
from s3transfer.futures import BoundedExecutor, TaskTag, TransferCoordinator, TransferFuture
from s3transfer.tasks import SubmissionTask as SubmissionTask
from s3transfer.tasks import Task as Task
from s3transfer.utils import S3_RETRYABLE_DOWNLOAD_ERRORS as S3_RETRYABLE_DOWNLOAD_ERRORS
from s3transfer.utils import CountCallbackInvoker as CountCallbackInvoker
from s3transfer.utils import DeferredOpenFile as DeferredOpenFile
from s3transfer.utils import FunctionContainer as FunctionContainer
from s3transfer.utils import OSUtils
from s3transfer.utils import StreamReaderProgress as StreamReaderProgress
from s3transfer.utils import calculate_num_parts as calculate_num_parts
from s3transfer.utils import calculate_range_parameter as calculate_range_parameter
from s3transfer.utils import get_callbacks as get_callbacks
from s3transfer.utils import invoke_progress_callbacks as invoke_progress_callbacks

_R = TypeVar("_R")

logger: logging.Logger

class DownloadOutputManager:
    def __init__(
        self,
        osutil: OSUtils,
        transfer_coordinator: TransferCoordinator,
        io_executor: BoundedExecutor,
    ) -> None: ...
    @classmethod
    def is_compatible(cls, download_target: IO[Any], osutil: OSUtils) -> bool: ...
    def get_download_task_tag(self) -> TaskTag: ...
    def get_fileobj_for_io_writes(self, transfer_future: TransferFuture) -> IO[Any]: ...
    def queue_file_io_task(
        self, fileobj: Union[IO[Any], str, bytes], data: str, offset: int
    ) -> None: ...
    def get_io_write_task(
        self, fileobj: Union[IO[Any], str, bytes], data: str, offset: int
    ) -> Task: ...
    def get_final_io_task(self) -> Task: ...

class DownloadFilenameOutputManager(DownloadOutputManager):
    def __init__(
        self,
        osutil: OSUtils,
        transfer_coordinator: TransferCoordinator,
        io_executor: BoundedExecutor,
    ) -> None: ...
    @classmethod
    def is_compatible(cls, download_target: IO[Any], osutil: OSUtils) -> bool: ...
    def get_fileobj_for_io_writes(self, transfer_future: TransferFuture) -> IO[Any]: ...
    def get_final_io_task(self) -> Task: ...

class DownloadSeekableOutputManager(DownloadOutputManager):
    @classmethod
    def is_compatible(cls, download_target: IO[Any], osutil: OSUtils) -> bool: ...
    def get_fileobj_for_io_writes(self, transfer_future: TransferFuture) -> IO[Any]: ...
    def get_final_io_task(self) -> Task: ...

class DownloadNonSeekableOutputManager(DownloadOutputManager):
    def __init__(
        self,
        osutil: OSUtils,
        transfer_coordinator: TransferCoordinator,
        io_executor: BoundedExecutor,
        defer_queue: Optional[DeferQueue] = ...,
    ) -> None: ...
    @classmethod
    def is_compatible(cls, download_target: IO[Any], osutil: OSUtils) -> bool: ...
    def get_download_task_tag(self) -> TaskTag: ...
    def get_fileobj_for_io_writes(self, transfer_future: TransferFuture) -> IO[Any]: ...
    def get_final_io_task(self) -> Task: ...
    def queue_file_io_task(
        self, fileobj: Union[IO[Any], str, bytes], data: str, offset: int
    ) -> None: ...
    def get_io_write_task(
        self, fileobj: Union[IO[Any], str, bytes], data: str, offset: int
    ) -> Task: ...

class DownloadSpecialFilenameOutputManager(DownloadNonSeekableOutputManager):
    def __init__(
        self,
        osutil: OSUtils,
        transfer_coordinator: TransferCoordinator,
        io_executor: BoundedExecutor,
        defer_queue: Optional[DeferQueue] = ...,
    ) -> None: ...
    @classmethod
    def is_compatible(cls, download_target: IO[Any], osutil: OSUtils) -> bool: ...
    def get_fileobj_for_io_writes(self, transfer_future: TransferFuture) -> IO[Any]: ...
    def get_final_io_task(self) -> Task: ...

class DownloadSubmissionTask(SubmissionTask): ...
class GetObjectTask(Task): ...
class ImmediatelyWriteIOGetObjectTask(GetObjectTask): ...
class IOWriteTask(Task): ...
class IOStreamingWriteTask(Task): ...
class IORenameFileTask(Task): ...
class IOCloseTask(Task): ...

class CompleteDownloadNOOPTask(Task):
    def __init__(
        self,
        transfer_coordinator: TransferCoordinator,
        main_kwargs: Optional[Dict[str, Any]] = ...,
        pending_main_kwargs: Optional[Dict[str, Any]] = ...,
        done_callbacks: Optional[Callable[..., Any]] = ...,
        is_final: bool = ...,
    ) -> None: ...

class DownloadChunkIterator:
    def __init__(self, body: IO[Any], chunksize: int) -> None: ...
    def __iter__(self: _R) -> _R: ...
    def __next__(self) -> str: ...
    next: Callable[[Any], Any]

class DeferQueue:
    def __init__(self) -> None: ...
    def request_writes(self, offset: int, data: str) -> List[Dict[str, Any]]: ...



import logging
from typing import IO, Any, List, Mapping, Optional, Sequence, Set, Type, TypeVar, Union

from botocore.client import BaseClient
from s3transfer.bandwidth import BandwidthLimiter as BandwidthLimiter
from s3transfer.bandwidth import LeakyBucket as LeakyBucket
from s3transfer.constants import ALLOWED_DOWNLOAD_ARGS as ALLOWED_DOWNLOAD_ARGS
from s3transfer.constants import KB as KB
from s3transfer.constants import MB as MB
from s3transfer.copies import CopySubmissionTask as CopySubmissionTask
from s3transfer.delete import DeleteSubmissionTask as DeleteSubmissionTask
from s3transfer.download import DownloadSubmissionTask as DownloadSubmissionTask
from s3transfer.exceptions import CancelledError as CancelledError
from s3transfer.exceptions import FatalError as FatalError
from s3transfer.futures import IN_MEMORY_DOWNLOAD_TAG as IN_MEMORY_DOWNLOAD_TAG
from s3transfer.futures import IN_MEMORY_UPLOAD_TAG as IN_MEMORY_UPLOAD_TAG
from s3transfer.futures import BaseExecutor
from s3transfer.futures import BoundedExecutor as BoundedExecutor
from s3transfer.futures import TransferCoordinator as TransferCoordinator
from s3transfer.futures import TransferFuture as TransferFuture
from s3transfer.futures import TransferMeta as TransferMeta
from s3transfer.subscribers import BaseSubscriber
from s3transfer.upload import UploadSubmissionTask as UploadSubmissionTask
from s3transfer.utils import CallArgs as CallArgs
from s3transfer.utils import OSUtils as OSUtils
from s3transfer.utils import SlidingWindowSemaphore as SlidingWindowSemaphore
from s3transfer.utils import TaskSemaphore as TaskSemaphore
from s3transfer.utils import get_callbacks as get_callbacks
from s3transfer.utils import signal_not_transferring as signal_not_transferring
from s3transfer.utils import signal_transferring as signal_transferring

_R = TypeVar("_R")

logger: logging.Logger

class TransferConfig:
    multipart_threshold: int
    multipart_chunksize: int
    max_request_concurrency: int
    max_submission_concurrency: int
    max_request_queue_size: int
    max_submission_queue_size: int
    max_io_queue_size: int
    io_chunksize: int
    num_download_attempts: int
    max_in_memory_upload_chunks: int
    max_in_memory_download_chunks: int
    max_bandwidth: int
    def __init__(
        self,
        multipart_threshold: int = ...,
        multipart_chunksize: int = ...,
        max_request_concurrency: int = ...,
        max_submission_concurrency: int = ...,
        max_request_queue_size: int = ...,
        max_submission_queue_size: int = ...,
        max_io_queue_size: int = ...,
        io_chunksize: int = ...,
        num_download_attempts: int = ...,
        max_in_memory_upload_chunks: int = ...,
        max_in_memory_download_chunks: int = ...,
        max_bandwidth: Optional[int] = ...,
    ) -> None: ...

class TransferManager:
    ALLOWED_DOWNLOAD_ARGS: List[str]
    ALLOWED_UPLOAD_ARGS: List[str]
    ALLOWED_COPY_ARGS: List[str]
    ALLOWED_DELETE_ARGS: List[str]
    VALIDATE_SUPPORTED_BUCKET_VALUES: bool
    def __init__(
        self,
        client: BaseClient,
        config: Optional[TransferConfig] = ...,
        osutil: Optional[OSUtils] = ...,
        executor_cls: Optional[Type[BaseExecutor]] = ...,
    ) -> None: ...
    @property
    def client(self) -> BaseClient: ...
    @property
    def config(self) -> TransferConfig: ...
    def upload(
        self,
        fileobj: Union[IO[Any], str, bytes],
        bucket: str,
        key: str,
        extra_args: Optional[Mapping[str, Any]] = ...,
        subscribers: Optional[Sequence[BaseSubscriber]] = ...,
    ) -> TransferFuture: ...
    def download(
        self,
        bucket: str,
        key: str,
        fileobj: Union[IO[Any], str, bytes],
        extra_args: Optional[Mapping[str, Any]] = ...,
        subscribers: Optional[Sequence[BaseSubscriber]] = ...,
    ) -> TransferFuture: ...
    def copy(
        self,
        copy_source: Mapping[str, Any],
        bucket: str,
        key: str,
        extra_args: Optional[Mapping[str, Any]] = ...,
        subscribers: Optional[Sequence[BaseSubscriber]] = ...,
        source_client: Optional[BaseClient] = ...,
    ) -> TransferFuture: ...
    def delete(
        self,
        bucket: str,
        key: str,
        extra_args: Optional[Mapping[str, Any]] = ...,
        subscribers: Optional[Sequence[BaseSubscriber]] = ...,
    ) -> TransferFuture: ...
    def __enter__(self: _R) -> _R: ...
    def __exit__(
        self, exc_type: Type[BaseException], exc_value: BaseException, *args: Any
    ) -> None: ...
    def shutdown(self, cancel: bool = ..., cancel_msg: str = ...) -> None: ...

class TransferCoordinatorController:
    def __init__(self) -> None: ...
    @property
    def tracked_transfer_coordinators(self) -> Set[TransferCoordinator]: ...
    def add_transfer_coordinator(self, transfer_coordinator: TransferCoordinator) -> None: ...
    def remove_transfer_coordinator(self, transfer_coordinator: TransferCoordinator) -> None: ...
    def cancel(self, msg: str = ..., exc_type: Type[BaseException] = ...) -> None: ...
    def wait(self) -> None: ...



from typing import IO, Any, Optional, TypeVar, Union

from s3transfer.futures import TransferCoordinator

_R = TypeVar("_R")

class RequestExceededException(Exception):
    def __init__(self, requested_amt: int, retry_time: float) -> None:
        self.requested_amt: int
        self.retry_time: float

class RequestToken: ...

class TimeUtils:
    def time(self) -> float: ...
    def sleep(self, value: float) -> None: ...

class BandwidthLimiter:
    def __init__(
        self, leaky_bucket: LeakyBucket, time_utils: Optional[TimeUtils] = ...
    ) -> None: ...
    def get_bandwith_limited_stream(
        self,
        fileobj: Union[IO[Any], str, bytes],
        transfer_coordinator: TransferCoordinator,
        enabled: bool = ...,
    ) -> BandwidthLimitedStream: ...

class BandwidthLimitedStream:
    def __init__(
        self,
        fileobj: Union[IO[Any], str, bytes],
        leaky_bucket: LeakyBucket,
        transfer_coordinator: TransferCoordinator,
        time_utils: Optional[TimeUtils] = ...,
        bytes_threshold: int = ...,
    ) -> None: ...
    def enable_bandwidth_limiting(self) -> None: ...
    def disable_bandwidth_limiting(self) -> None: ...
    def read(self, amount: int) -> str: ...
    def signal_transferring(self) -> None: ...
    def signal_not_transferring(self) -> None: ...
    def seek(self, where: int, whence: int = ...) -> None: ...
    def tell(self) -> int: ...
    def close(self) -> None: ...
    def __enter__(self: _R) -> _R: ...
    def __exit__(self, *args: Any, **kwargs: Any) -> None: ...

class LeakyBucket:
    def __init__(
        self,
        max_rate: int,
        time_utils: Optional[TimeUtils] = ...,
        rate_tracker: Optional[BandwidthRateTracker] = ...,
        consumption_scheduler: Optional[ConsumptionScheduler] = ...,
    ) -> None: ...
    def consume(self, amt: int, request_token: RequestToken) -> int: ...

class ConsumptionScheduler:
    def __init__(self) -> None: ...
    def is_scheduled(self, token: RequestToken) -> bool: ...
    def schedule_consumption(
        self, amt: int, token: RequestToken, time_to_consume: float
    ) -> float: ...
    def process_scheduled_consumption(self, token: RequestToken) -> None: ...

class BandwidthRateTracker:
    def __init__(self, alpha: float = ...) -> None: ...
    @property
    def current_rate(self) -> float: ...
    def get_projected_rate(self, amt: int, time_at_consumption: float) -> float: ...
    def record_consumption_rate(self, amt: int, time_at_consumption: float) -> None: ...



import logging
import multiprocessing
from collections.abc import Generator
from queue import Queue
from typing import Any, Dict, Mapping, NamedTuple, Optional, Type, TypeVar

from botocore.client import BaseClient
from s3transfer.compat import MAXINT as MAXINT
from s3transfer.compat import BaseManager as BaseManager
from s3transfer.constants import ALLOWED_DOWNLOAD_ARGS as ALLOWED_DOWNLOAD_ARGS
from s3transfer.constants import MB as MB
from s3transfer.constants import PROCESS_USER_AGENT as PROCESS_USER_AGENT
from s3transfer.exceptions import CancelledError as CancelledError
from s3transfer.exceptions import RetriesExceededError as RetriesExceededError
from s3transfer.futures import BaseTransferFuture as BaseTransferFuture
from s3transfer.futures import BaseTransferMeta as BaseTransferMeta
from s3transfer.futures import TransferFuture
from s3transfer.manager import TransferConfig
from s3transfer.utils import S3_RETRYABLE_DOWNLOAD_ERRORS as S3_RETRYABLE_DOWNLOAD_ERRORS
from s3transfer.utils import CallArgs as CallArgs
from s3transfer.utils import OSUtils as OSUtils
from s3transfer.utils import calculate_num_parts as calculate_num_parts
from s3transfer.utils import calculate_range_parameter as calculate_range_parameter

_R = TypeVar("_R")

logger: logging.Logger
SHUTDOWN_SIGNAL: str

class DownloadFileRequest(NamedTuple):
    transfer_id: str
    bucket: str
    key: str
    filename: str
    extra_args: Dict[str, Any]
    expected_size: int

class GetObjectJob(NamedTuple):
    transfer_id: str
    bucket: str
    key: str
    temp_filename: str
    extra_args: Dict[str, Any]
    offset: int
    filename: str

def ignore_ctrl_c() -> Generator[None, None, None]: ...

class ProcessTransferConfig:
    multipart_threshold: int
    multipart_chunksize: int
    max_request_processes: int
    def __init__(
        self,
        multipart_threshold: int = ...,
        multipart_chunksize: int = ...,
        max_request_processes: int = ...,
    ) -> None: ...

class ProcessPoolDownloader:
    def __init__(
        self,
        client_kwargs: Optional[Mapping[str, Any]] = ...,
        config: Optional[ProcessTransferConfig] = ...,
    ) -> None: ...
    def download_file(
        self,
        bucket: str,
        key: str,
        filename: str,
        extra_args: Optional[Mapping[str, Any]] = ...,
        expected_size: Optional[int] = ...,
    ) -> TransferFuture: ...
    def shutdown(self) -> None: ...
    def __enter__(self: _R) -> _R: ...
    def __exit__(
        self, exc_type: Type[BaseException], exc_value: BaseException, *args: Any
    ) -> None: ...

class ProcessPoolTransferFuture(BaseTransferFuture):
    def __init__(self, monitor: TransferMonitor, meta: ProcessPoolTransferMeta) -> None: ...
    @property
    def meta(self) -> ProcessPoolTransferMeta: ...
    def done(self) -> bool: ...
    def result(self) -> str: ...
    def cancel(self) -> None: ...

class ProcessPoolTransferMeta(BaseTransferMeta):
    def __init__(self, transfer_id: str, call_args: Mapping[str, Any]) -> None: ...
    @property
    def call_args(self) -> Dict[str, Any]: ...
    @property
    def transfer_id(self) -> str: ...
    @property
    def user_context(self) -> Dict[str, Any]: ...

class ClientFactory:
    def __init__(self, client_kwargs: Optional[Mapping[str, Any]] = ...) -> None: ...
    def create_client(self) -> BaseClient: ...

class TransferMonitor:
    def __init__(self) -> None: ...
    def notify_new_transfer(self) -> str: ...
    def is_done(self, transfer_id: str) -> bool: ...
    def notify_done(self, transfer_id: str) -> None: ...
    def poll_for_result(self, transfer_id: str) -> None: ...
    def notify_exception(self, transfer_id: str, exception: BaseException) -> None: ...
    def notify_cancel_all_in_progress(self) -> None: ...
    def get_exception(self, transfer_id: str) -> Optional[BaseException]: ...
    def notify_expected_jobs_to_complete(self, transfer_id: str, num_jobs: int) -> None: ...
    def notify_job_complete(self, transfer_id: str) -> int: ...

class TransferState:
    def __init__(self) -> None: ...
    @property
    def done(self) -> bool: ...
    def set_done(self) -> None: ...
    def wait_till_done(self) -> None: ...
    @property
    def exception(self) -> Optional[BaseException]: ...
    @exception.setter
    def exception(self, val: BaseException) -> None: ...
    @property
    def jobs_to_complete(self) -> int: ...
    @jobs_to_complete.setter
    def jobs_to_complete(self, val: int) -> None: ...
    def decrement_jobs_to_complete(self) -> int: ...

class TransferMonitorManager(BaseManager): ...

class BaseS3TransferProcess(multiprocessing.Process):
    def __init__(self, client_factory: ClientFactory) -> None: ...
    def run(self) -> None: ...

class GetObjectSubmitter(BaseS3TransferProcess):
    def __init__(
        self,
        transfer_config: TransferConfig,
        client_factory: ClientFactory,
        transfer_monitor: TransferMonitor,
        osutil: OSUtils,
        download_request_queue: Queue[Any],
        worker_queue: Queue[Any],
    ) -> None: ...

class GetObjectWorker(BaseS3TransferProcess):
    def __init__(
        self,
        queue: Queue[Any],
        client_factory: ClientFactory,
        transfer_monitor: TransferMonitor,
        osutil: OSUtils,
    ) -> None: ...



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['s3transfer-stubs']

package_data = \
{'': ['*']}

install_requires = \
['pip']

setup_kwargs = {
    'name': 'types-s3transfer',
    'version': '0.6.0.post2',
    'description': 'Type annotations and code completion for s3transfer',
    'long_description': '# types-s3transfer\n\n[![PyPI - types-s3transfer](https://img.shields.io/pypi/v/types-s3transfer.svg?color=blue)](https://pypi.org/project/types-s3transfer)\n[![PyPI - Python Version](https://img.shields.io/pypi/pyversions/types-s3transfer.svg?color=blue)](https://pypi.org/project/types-s3transfer)\n[![PyPI - Downloads](https://img.shields.io/pypi/dm/types-s3transfer?color=blue)](https://pypistats.org/packages/types-s3transfer)\n\n![boto3.typed](https://github.com/youtype/mypy_boto3_builder/raw/main/logo.png)\n\nType annotations and code completion for [s3transfer](https://pypi.org/project/s3transfer/) package.\nThis package is a part of [mypy_boto3_builder](https://github.com/youtype/mypy_boto3_builder) project.\n\n## Installation\n\n```bash\npython -m pip install types-s3transfer\n```\n\n## Usage\n\nUse [mypy](https://github.com/python/mypy) or [pyright](https://github.com/microsoft/pyright) for type checking.\n\n### Latest changes\n\nBuilder changelog can be found in [Releases](https://github.com/youtype/mypy_boto3_builder/releases).\n\n## Versioning\n\n`types-s3transfer` version is the same as related `s3transfer` version and follows\n[PEP 440](https://www.python.org/dev/peps/pep-0440/) format.\n\n## Support and contributing\n\nPlease reports any bugs or request new features in\n[mypy-boto3-builder](https://github.com/youtype/mypy_boto3_builder/issues/) repository.\n',
    'author': 'Vlad Emelianov',
    'author_email': 'vlad.emelianov.nz@gmail.com',
    'maintainer': None,
    'maintainer_email': None,
    'url': 'https://youtype.github.io/mypy_boto3_builder/',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'python_requires': '>=3.6,<4.0',
}


setup(**setup_kwargs)



import logging
from typing import Any, Callable, Dict, Optional, Sequence

from s3transfer.futures import TransferCoordinator
from s3transfer.utils import get_callbacks as get_callbacks

logger: logging.Logger

class Task:
    def __init__(
        self,
        transfer_coordinator: TransferCoordinator,
        main_kwargs: Optional[Dict[str, Any]] = ...,
        pending_main_kwargs: Optional[Dict[str, Any]] = ...,
        done_callbacks: Optional[Sequence[Callable[..., Any]]] = ...,
        is_final: bool = ...,
    ) -> None: ...
    @property
    def transfer_id(self) -> str: ...
    def __call__(self) -> None: ...

class SubmissionTask(Task): ...
class CreateMultipartUploadTask(Task): ...
class CompleteMultipartUploadTask(Task): ...
